Package: caret
Title: Classification and Regression Training
Version: 7.0-1
Authors@R: 
    c(person(given = "Max",
             family = "Kuhn",
             role = c("aut", "cre"),
             email = "mxkuhn@gmail.com",
             comment = c(ORCID = "0000-0003-2402-136X")),
      person(given = "Jed",
             family = "Wing",
             role = "ctb"),
      person(given = "Steve",
             family = "Weston",
             role = "ctb"),
      person(given = "Andre",
             family = "Williams",
             role = "ctb"),
      person(given = "Chris",
             family = "Keefer",
             role = "ctb"),
      person(given = "Allan",
             family = "Engelhardt",
             role = "ctb"),
      person(given = "Tony",
             family = "Cooper",
             role = "ctb"),
      person(given = "Zachary",
             family = "Mayer",
             role = "ctb"),
      person(given = "Brenton",
             family = "Kenkel",
             role = "ctb"),
      person(given = "R Core Team",
             role = "ctb"),
      person(given = "Michael",
             family = "Benesty",
             role = "ctb"),
      person(given = "Reynald",
             family = "Lescarbeau",
             role = "ctb"),
      person(given = "Andrew",
             family = "Ziem",
             role = "ctb"),
      person(given = "Luca",
             family = "Scrucca",
             role = "ctb"),
      person(given = "Yuan",
             family = "Tang",
             role = "ctb"),
      person(given = "Can",
             family = "Candan",
             role = "ctb"),
      person(given = "Tyler",
             family = "Hunt",
             role = "ctb"))
Description: Misc functions for training and plotting classification and
    regression models.
License: GPL (>= 2)
URL: https://github.com/topepo/caret/
BugReports: https://github.com/topepo/caret/issues
Depends: ggplot2, lattice (>= 0.20), R (>= 3.2.0)
Imports: e1071, foreach, grDevices, methods, ModelMetrics (>= 1.2.2.2),
        nlme, plyr, pROC, recipes (>= 0.1.10), reshape2, stats, stats4,
        utils, withr (>= 2.0.0)
Suggests: BradleyTerry2, covr, Cubist, dplyr, earth (>= 2.2-3),
        ellipse, fastICA, gam (>= 1.15), ipred, kernlab, klaR, knitr,
        MASS, Matrix, mda, mgcv, mlbench, MLmetrics, nnet, pamr, party
        (>= 0.9-99992), pls, proxy, randomForest, RANN, rmarkdown,
        rpart, spls, superpc, testthat (>= 0.9.1), themis (>= 0.1.3)
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2024-12-09 16:28:32 UTC; max
Author: Max Kuhn [aut, cre] (<https://orcid.org/0000-0003-2402-136X>),
  Jed Wing [ctb],
  Steve Weston [ctb],
  Andre Williams [ctb],
  Chris Keefer [ctb],
  Allan Engelhardt [ctb],
  Tony Cooper [ctb],
  Zachary Mayer [ctb],
  Brenton Kenkel [ctb],
  R Core Team [ctb],
  Michael Benesty [ctb],
  Reynald Lescarbeau [ctb],
  Andrew Ziem [ctb],
  Luca Scrucca [ctb],
  Yuan Tang [ctb],
  Can Candan [ctb],
  Tyler Hunt [ctb]
Maintainer: Max Kuhn <mxkuhn@gmail.com>
Repository: CRAN
Date/Publication: 2024-12-10 01:30:01 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 13:32:50 UTC; unix
Archs: caret.so.dSYM
