# Generated by roxygen2: do not edit by hand

S3method(BoxCoxTrans,default)
S3method(F_meas,default)
S3method(F_meas,table)
S3method(as.data.frame,resamples)
S3method(as.matrix,confusionMatrix)
S3method(as.matrix,resamples)
S3method(as.table,confusionMatrix)
S3method(avNNet,default)
S3method(avNNet,formula)
S3method(bag,default)
S3method(bag,formula)
S3method(bagEarth,default)
S3method(bagEarth,formula)
S3method(bagFDA,default)
S3method(bagFDA,formula)
S3method(bwplot,diff.resamples)
S3method(bwplot,resamples)
S3method(calibration,default)
S3method(calibration,formula)
S3method(classDist,default)
S3method(cluster,default)
S3method(cluster,resamples)
S3method(confusionMatrix,default)
S3method(confusionMatrix,matrix)
S3method(confusionMatrix,rfe)
S3method(confusionMatrix,sbf)
S3method(confusionMatrix,table)
S3method(confusionMatrix,train)
S3method(densityplot,diff.resamples)
S3method(densityplot,resamples)
S3method(densityplot,rfe)
S3method(densityplot,sbf)
S3method(densityplot,train)
S3method(diff,resamples)
S3method(dotplot,diff.resamples)
S3method(dotplot,resamples)
S3method(dummyVars,default)
S3method(enumLC,default)
S3method(enumLC,formula)
S3method(enumLC,lm)
S3method(enumLC,matrix)
S3method(expoTrans,default)
S3method(expoTrans,numeric)
S3method(fitted,train)
S3method(format,bagEarth)
S3method(gafs,default)
S3method(gafs,recipe)
S3method(get_resample_perf,gafs)
S3method(get_resample_perf,rfe)
S3method(get_resample_perf,safs)
S3method(get_resample_perf,sbf)
S3method(get_resample_perf,train)
S3method(ggplot,calibration)
S3method(ggplot,gafs)
S3method(ggplot,lift)
S3method(ggplot,resamples)
S3method(ggplot,rfe)
S3method(ggplot,safs)
S3method(ggplot,train)
S3method(ggplot,varImp.train)
S3method(histogram,rfe)
S3method(histogram,sbf)
S3method(histogram,train)
S3method(icr,default)
S3method(icr,formula)
S3method(knn3,data.frame)
S3method(knn3,default)
S3method(knn3,formula)
S3method(knn3,matrix)
S3method(knnreg,data.frame)
S3method(knnreg,default)
S3method(knnreg,formula)
S3method(knnreg,matrix)
S3method(levelplot,diff.resamples)
S3method(levels,train)
S3method(lift,default)
S3method(lift,formula)
S3method(negPredValue,default)
S3method(negPredValue,matrix)
S3method(negPredValue,table)
S3method(nullModel,default)
S3method(oob_pred,list)
S3method(oob_pred,rfe)
S3method(oob_pred,sbf)
S3method(oob_pred,train)
S3method(parallelplot,resamples)
S3method(pcaNNet,default)
S3method(pcaNNet,formula)
S3method(plot,calibration)
S3method(plot,gafs)
S3method(plot,lift)
S3method(plot,prcomp.resamples)
S3method(plot,rfe)
S3method(plot,safs)
S3method(plot,train)
S3method(plot,varImp.train)
S3method(plsda,default)
S3method(posPredValue,default)
S3method(posPredValue,matrix)
S3method(posPredValue,table)
S3method(prcomp,resamples)
S3method(preProcess,default)
S3method(precision,default)
S3method(precision,table)
S3method(predict,BoxCoxTrans)
S3method(predict,PLS)
S3method(predict,avNNet)
S3method(predict,bag)
S3method(predict,bagEarth)
S3method(predict,bagFDA)
S3method(predict,classDist)
S3method(predict,dummyVars)
S3method(predict,expoTrans)
S3method(predict,gafs)
S3method(predict,icr)
S3method(predict,knn3)
S3method(predict,knnreg)
S3method(predict,list)
S3method(predict,nullModel)
S3method(predict,pcaNNet)
S3method(predict,plsda)
S3method(predict,preProcess)
S3method(predict,rfe)
S3method(predict,safs)
S3method(predict,sbf)
S3method(predict,splsda)
S3method(predict,train)
S3method(predict,train.recipe)
S3method(predictors,default)
S3method(predictors,formula)
S3method(predictors,gafs)
S3method(predictors,list)
S3method(predictors,rfe)
S3method(predictors,safs)
S3method(predictors,sbf)
S3method(predictors,terms)
S3method(predictors,train)
S3method(print,BoxCoxTrans)
S3method(print,avNNet)
S3method(print,bag)
S3method(print,bagEarth)
S3method(print,bagFDA)
S3method(print,bmars)
S3method(print,calibration)
S3method(print,classDist)
S3method(print,confusionMatrix)
S3method(print,confusionMatrix.rfe)
S3method(print,confusionMatrix.sbf)
S3method(print,confusionMatrix.train)
S3method(print,diff.resamples)
S3method(print,dummyVars)
S3method(print,expoTrans)
S3method(print,gafs)
S3method(print,icr)
S3method(print,knn3)
S3method(print,knnreg)
S3method(print,lift)
S3method(print,nullModel)
S3method(print,pcaNNet)
S3method(print,plsda)
S3method(print,prcomp.resamples)
S3method(print,preProcess)
S3method(print,resamples)
S3method(print,rfe)
S3method(print,safs)
S3method(print,sbf)
S3method(print,splsda)
S3method(print,summary.bag)
S3method(print,summary.bagEarth)
S3method(print,summary.bagFDA)
S3method(print,summary.diff.resamples)
S3method(print,summary.resamples)
S3method(print,train)
S3method(print,varImp.train)
S3method(recall,default)
S3method(recall,table)
S3method(resamples,default)
S3method(residuals,train)
S3method(rfe,default)
S3method(rfe,formula)
S3method(rfe,recipe)
S3method(safs,default)
S3method(safs,recipe)
S3method(sbf,default)
S3method(sbf,formula)
S3method(sbf,recipe)
S3method(sensitivity,default)
S3method(sensitivity,matrix)
S3method(sensitivity,table)
S3method(sort,resamples)
S3method(spatialSign,data.frame)
S3method(spatialSign,default)
S3method(spatialSign,matrix)
S3method(specificity,default)
S3method(specificity,matrix)
S3method(specificity,table)
S3method(splom,resamples)
S3method(splsda,default)
S3method(stripplot,rfe)
S3method(stripplot,train)
S3method(summary,bag)
S3method(summary,bagEarth)
S3method(summary,bagFDA)
S3method(summary,diff.resamples)
S3method(summary,resamples)
S3method(summary,train)
S3method(train,default)
S3method(train,formula)
S3method(train,recipe)
S3method(trim,train)
S3method(update,gafs)
S3method(update,rfe)
S3method(update,safs)
S3method(update,train)
S3method(varImp,C5.0)
S3method(varImp,Gam)
S3method(varImp,JRip)
S3method(varImp,PART)
S3method(varImp,RRF)
S3method(varImp,RandomForest)
S3method(varImp,avNNet)
S3method(varImp,bagEarth)
S3method(varImp,bagFDA)
S3method(varImp,classbagg)
S3method(varImp,cubist)
S3method(varImp,dsa)
S3method(varImp,earth)
S3method(varImp,fda)
S3method(varImp,gafs)
S3method(varImp,gam)
S3method(varImp,gbm)
S3method(varImp,glm)
S3method(varImp,glmnet)
S3method(varImp,lm)
S3method(varImp,multinom)
S3method(varImp,mvr)
S3method(varImp,nnet)
S3method(varImp,pamrtrained)
S3method(varImp,plsda)
S3method(varImp,randomForest)
S3method(varImp,regbagg)
S3method(varImp,rfe)
S3method(varImp,rpart)
S3method(varImp,safs)
S3method(varImp,sbf)
S3method(varImp,train)
S3method(xyplot,calibration)
S3method(xyplot,lift)
S3method(xyplot,resamples)
S3method(xyplot,rfe)
S3method(xyplot,train)
export(BoxCoxTrans)
export(F_meas)
export(LPH07_1)
export(LPH07_2)
export(MAE)
export(MeanSD)
export(R2)
export(RMSE)
export(SLC14_1)
export(SLC14_2)
export(anovaScores)
export(avNNet)
export(bag)
export(bagControl)
export(bagEarth)
export(bagEarthStats)
export(bagFDA)
export(best)
export(calibration)
export(caretFuncs)
export(caretGA)
export(caretSA)
export(caretSBF)
export(caretTheme)
export(cforestStats)
export(checkConditionalX)
export(checkInstall)
export(checkResamples)
export(class2ind)
export(classDist)
export(cluster)
export(compare_models)
export(confusionMatrix)
export(confusionMatrix.train)
export(contr.dummy)
export(contr.ltfr)
export(createDataPartition)
export(createFolds)
export(createModel)
export(createMultiFolds)
export(createResample)
export(createTimeSlices)
export(ctreeBag)
export(defaultSummary)
export(dotPlot)
export(downSample)
export(dummyVars)
export(expandParameters)
export(expoTrans)
export(extractPrediction)
export(extractProb)
export(featurePlot)
export(filterVarImp)
export(findCorrelation)
export(findLinearCombos)
export(flatTable)
export(gafs)
export(gafs.default)
export(gafsControl)
export(gafs_initial)
export(gafs_lrSelection)
export(gafs_raMutation)
export(gafs_rwSelection)
export(gafs_spCrossover)
export(gafs_tourSelection)
export(gafs_uCrossover)
export(gamFormula)
export(gamFuncs)
export(gamScores)
export(getModelInfo)
export(getSamplingInfo)
export(getTrainPerf)
export(ggplot.gafs)
export(ggplot.safs)
export(groupKFold)
export(hasTerms)
export(icr)
export(index2vec)
export(ipredStats)
export(knn3)
export(knn3Train)
export(knnreg)
export(knnregTrain)
export(ldaBag)
export(ldaFuncs)
export(ldaSBF)
export(learning_curve_dat)
export(lift)
export(lmFuncs)
export(lmSBF)
export(lrFuncs)
export(maxDissim)
export(minDiss)
export(mnLogLoss)
export(modelCor)
export(modelLookup)
export(multiClassSummary)
export(nbBag)
export(nbFuncs)
export(nbSBF)
export(nearZeroVar)
export(negPredValue)
export(nnetBag)
export(nullModel)
export(nzv)
export(oneSE)
export(outcome_conversion)
export(panel.calibration)
export(panel.lift)
export(panel.lift2)
export(panel.needle)
export(pcaNNet)
export(pickSizeBest)
export(pickSizeTolerance)
export(pickVars)
export(plot.gafs)
export(plot.rfe)
export(plot.safs)
export(plot.train)
export(plotClassProbs)
export(plotObsVsPred)
export(plsBag)
export(plsda)
export(posPredValue)
export(postResample)
export(prSummary)
export(preProcess)
export(precision)
export(predict.bagEarth)
export(predict.gafs)
export(predict.train)
export(predictionFunction)
export(predictors)
export(print.train)
export(probFunction)
export(progress)
export(recall)
export(resampleHist)
export(resampleSummary)
export(resampleWrapper)
export(resamples)
export(rfFuncs)
export(rfGA)
export(rfSA)
export(rfSBF)
export(rfStats)
export(rfe)
export(rfeControl)
export(rfeIter)
export(safs)
export(safsControl)
export(safs_initial)
export(safs_perturb)
export(safs_prob)
export(sbf)
export(sbfControl)
export(sbfIter)
export(sensitivity)
export(sortImp)
export(spatialSign)
export(specificity)
export(splsda)
export(sumDiss)
export(summary.bagEarth)
export(svmBag)
export(thresholder)
export(tolerance)
export(train)
export(trainControl)
export(treebagFuncs)
export(treebagGA)
export(treebagSA)
export(treebagSBF)
export(twoClassSim)
export(twoClassSummary)
export(upSample)
export(varImp)
export(var_seq)
export(well_numbered)
import(foreach)
import(ggplot2)
import(lattice)
import(methods)
import(nlme)
import(plyr)
import(recipes)
import(reshape2)
importFrom(ModelMetrics,auc)
importFrom(grDevices,extendrange)
importFrom(pROC,roc)
importFrom(plyr,ddply)
importFrom(recipes,all_outcomes)
importFrom(recipes,all_predictors)
importFrom(recipes,bake)
importFrom(recipes,has_role)
importFrom(recipes,juice)
importFrom(recipes,prep)
importFrom(stats,.checkMFClasses)
importFrom(stats,.getXlevels)
importFrom(stats,aggregate)
importFrom(stats,anova)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,binom.test)
importFrom(stats,binomial)
importFrom(stats,complete.cases)
importFrom(stats,contrasts)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,delete.response)
importFrom(stats,dist)
importFrom(stats,fitted)
importFrom(stats,fitted.values)
importFrom(stats,glm)
importFrom(stats,hclust)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,mahalanobis)
importFrom(stats,mcnemar.test)
importFrom(stats,median)
importFrom(stats,model.extract)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.response)
importFrom(stats,model.weights)
importFrom(stats,na.fail)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,optim)
importFrom(stats,p.adjust)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,rbinom)
importFrom(stats,reorder)
importFrom(stats,reshape)
importFrom(stats,resid)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,t.test)
importFrom(stats,terms)
importFrom(stats,toeplitz)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats4,coef)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,flush.console)
importFrom(utils,getFromNamespace)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,menu)
importFrom(utils,modifyList)
importFrom(utils,object.size)
importFrom(utils,stack)
importFrom(withr,with_seed)
useDynLib(caret)
